package atomicwheel.util;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Set;

/**
 * {@link HashMap} that maps a key to a {@link Collection} of values.
 * 
 * @param <K>
 *            type of the keys
 * @param <V>
 *            type of the values
 * @param <Coll>
 *            type of the collection containing the values
 * 
 * @author matthias
 */
public abstract class CollectionMap<K, V, Coll extends Collection<V>>
		implements Serializable {
	private static final long serialVersionUID = 1L;

	protected final HashMap<K, Coll> map = new HashMap<K, Coll>();

	/**
	 * Adds the given value to the collection for the given key.
	 * 
	 * @param key
	 *            key to collection
	 * @param value
	 *            value to add
	 */
	public void add(final K key, final V value) {
		get(key).add(value);
	}

	/**
	 * Removes the given value from the collection for the given key.
	 * 
	 * @param key
	 *            key to collection
	 * @param value
	 *            value to add
	 */
	public void remove(final K key, final V value) {
		if (!map.containsKey(key))
			return;
		map.get(key).remove(value);
	}

	/**
	 * Adds the given values to the collection for the given key.
	 * 
	 * @param key
	 *            key to collection
	 * @param values
	 *            values to add
	 */
	public void addAll(final K key, final Collection<V> values) {
		get(key).addAll(values);
	}

	/**
	 * Removes the given value from the collection for the given key.
	 * 
	 * @param key
	 *            key to collection
	 * @param value
	 *            value to add
	 */
	public void removeAll(final K key, final Collection<V> values) {
		if (!map.containsKey(key))
			return;
		map.get(key).removeAll(values);
	}

	/**
	 * Gets the collection for the given key. Returns the internal collection.
	 * 
	 * @param key
	 *            the key to query
	 * @return its collection (may be empty but not null)
	 */
	public Coll get(final K key) {
		if (!map.containsKey(key))
			map.put(key, createCollection());
		return map.get(key);
	}

	/**
	 * Get union of the values for all given keys. Union is defined as the way
	 * the collection handles addAll. Returns a new collection, so modifications
	 * do not get reflected in the {@link CollectionMap}.
	 * 
	 * @param keys
	 *            keys to query
	 * @return the union of all values for these keys
	 */
	public Coll getAll(final Collection<K> keys) {
		final Coll list = createCollection();
		for (final K key : keys)
			list.addAll(get(key));
		return list;
	}

	/**
	 * Gets a set of all keys.
	 * 
	 * @return a set of all keys
	 */
	public Set<K> keySet() {
		return map.keySet();
	}

	/**
	 * Creates a new instance of the collection.
	 * 
	 * @return <code>new Coll()</code>
	 */
	protected abstract Coll createCollection();
}
